/*
	Copyright  1995-2001, The AROS Development Team. All rights reserved.
	$Id: launchtoola.c,v 1.1.1.1 2005/03/15 15:57:28 laire Exp $

	Desc:
	Lang: English
*/
#include <datatypes/datatypesclass.h>
#include <utility/tagitem.h>
#include <dos/dostags.h>
#include <proto/alib.h>
#include <proto/dos.h>
#include <proto/exec.h>
#include <proto/utility.h>
#include "datatypes_intern.h"
#include <workbench/startup.h>
#include <proto/wbstart.h>

/*****************************************************************************

	NAME */
#include <proto/datatypes.h>

	AROS_LH3(ULONG, LaunchToolA,

/*  SYNOPSIS */
	AROS_LHA(struct Tool *   , tool,    A0),
		AROS_LHA(STRPTR          , project, A1),
	AROS_LHA(struct TagItem *, attrs,   A2),

/*  LOCATION */
	struct Library *, DataTypesBase, 42, DataTypes)

/*  FUNCTION

	Launch an application with a particular project.

	INPUTS

	tool     --  tool to use (may be NULL in which case this function
				 returns 0)
	project  --  name of the project to execute or NULL
	attrs    --  additional attributes

	TAGS

	NP_Priority (BYTE) -- priority of the launched tool (default is the
						  priority of the currect process except for
			  Workbench applications where the default priority
			  is 0 if not overridden by the TOOLPRI tooltype).
			  
	NP_Synchronous (BOOL) -- don't return until lauched application process
							 finishes (defaults to FALSE).

	RESULT

	Zero for failure, non-zero otherwise.

	NOTES

	EXAMPLE

	BUGS

	SEE ALSO

	NewDTObjectA()

	INTERNALS

	HISTORY

	SDuvan  17.12.2000  --  basic implementation

*****************************************************************************/
{
	AROS_LIBFUNC_INIT

	STRPTR tBuffer;
	STRPTR tStr = NULL;
	BOOL isSynchronous;
	BPTR output;
	LONG ret;

	if (tool == NULL)
	{
	SetIoErr(ERROR_REQUIRED_ARG_MISSING);
	
	return 0;
	}

	isSynchronous = (BOOL)GetTagData(NP_Synchronous, (IPTR)FALSE, attrs);

	switch (tool->tn_Flags & TF_LAUNCH_MASK)
	{
	case TF_RX:
	    tStr = ((project) ? "RX %s \"%s\"" : "RX %s");
	case TF_SHELL:
	    if (!tStr)
		tStr = ((project) ? "%s \"%s\"" : "%s");

	    output = Open("CON:////Output window/AUTO/WAIT/CLOSE/INACTIVE",
			  MODE_NEWFILE);

	    if (output != NULL)
	    {
		struct TagItem tags[] = { { SYS_Asynch, !isSynchronous },
					  { SYS_Input , NULL           },
					  { SYS_Output, (IPTR)output   },
					  { TAG_DONE, NULL             } };

		tBuffer = AllocVec(strlen(tStr) + strlen(tool->tn_Program) + ((project) ? strlen(project) : 0), MEMF_PUBLIC);

		dt_sprintf(DataTypesBase, tBuffer,
		    tStr, tool->tn_Program, project);

		ret = SystemTagList(tBuffer, tags);

		FreeVec(tBuffer);

		/* Error? */
		if (ret == -1)
		{
		    return 0;
		}

		Close(output);
	    }
	    else
	    {
		return 0;
	    }
	break;

	case TF_WORKBENCH:
	{
		struct Library *WBStartBase;
		ULONG rc = 0;

		if ((WBStartBase = OpenLibrary("wbstart.library", 2)))
		{
			struct WBArg wba[1];
			BYTE pri;

			/* This casting is deliberate to get correct sign for both LONG and BYTE, *DO NOT CHANGE*! */
			pri = (LONG)GetTagData(NP_Priority, 0, attrs);

			if (project)
			{
				if ((wba[0].wa_Lock = Lock(project, ACCESS_READ)))
				{
					wba[0].wa_Name = project;

					rc = WBStartTags(WBStart_Name, (ULONG)tool->tn_Program,
									 WBStart_Priority, pri,
									 WBStart_ArgumentCount, 1,
									 WBStart_ArgumentList, (ULONG)wba,
									 TAG_DONE);

					UnLock(wba[0].wa_Lock);
				}
			}
			else
			{
				rc = WBStartTags(WBStart_Name, (ULONG)tool->tn_Program,
								 WBStart_Priority, pri,
								 TAG_DONE);
			}
			CloseLibrary(WBStartBase);
		}
		return (rc);
	}
	break;

	default:
	return 0;
	break;
	}

	return 1;
	
	AROS_LIBFUNC_EXIT
} /* LaunchToolA */

